
#L6/2nd Prac: Part A Simulation and Statistical Programming

#rm(list=ls())

##
#functions
sin(pi)/pi

f<-function(x) {
	#sinc function
	a=sin(x)/x
	return(a)	
}

f(pi)
f(c(-pi,0,pi)) #our functions may work on vectors etc.
f(pi/2)-2/pi   #pi/2 is evaluated

#equivalent formats
f<-function(x) {
	return(sin(x)/x) #evaluate then return
}

f<-function(x) {
	a=sin(x)/x
	a			#{...} evaluates to give last expression anyway	       
}

f<-function(x) {
	sin(x)/x        
}

#for one-statement functions only, dont need {} 
f<-function(x) sin(x)/x 
 
#fix Nan at zero and plot
f<-function(x) {
	i=which(x==0)
	a=sin(x)/x
	a[i]=1
	return(a)		 
}
curve(sin(x),from=-4*pi,to=4*pi)
curve(f(x),from=-6*pi,to=6*pi,add=TRUE,col=2)

#multiple arguments, defaults 
g<-function(x,p=2) {
	y=abs(x)
      z=sum(y^p)
	z^(1/p)
}
v=c(3,4)
g(v)		#Euclidean norm
g(v,1)	#Manhattan norm
g(v,p=1)    #reads nicely

##
#basics of scope
f<-function(x) {
	a=b*x^2
	return(a)
}
a=2
b=1
f(5)
a

##
#for loops
for (i in 1:10) print(i^2)

fibonacci = function(n) {
	#evalute first n terms in Fibonacci sequence
	x = numeric(n)
	x[1:2] = 1
	for (i in 3:n) x[i] <- x[i-2] + x[i-1]
	return(x)
}

#can build up vector by concatenating last result
fibonacci = function(n) {
	#illustrating concatenation
	x = c(1,1)
	for (i in 1:(n-2)) {
		a=x[i] + x[i+1]
		x=c(x,a)
	}
	return(x)
}

##
#if else
x=1
if (x > 2) {
    y = 2 * x
} else {
    y = 3*x
}
y
 
#gives a warning if x=1
z=(x-1)/(y-3)
if (is.nan(z)) warning('z is not a number')

##
#while
Z<-function(a) {
      z=rnorm(1) #rnorm(1) simulates 1 N(0,1) rv
	while (z<a) {
		z=rnorm(1)
	}
	z
}
Z(4) #careful!

pnorm(4)   #cdf Pr(Z<4)
1/(1-pnorm(4))

##
#returning several function outputs
Z<-function(a) {
	count=1
	z=rnorm(1)
	while (z<a) {
		z=rnorm(1)
		count=count+1
	}
	return(list(value=z,trials=count))
}
Z(4)
sim<-Z(4)
sim$value
sim[[1]]

##
#Newton Raphson
f <- function(x) {x^3 + 2*x^2 - 7}
f.prime <- function(x) {3*x^2 + 4*x}

curve(f, from=-5, to=5,main='f(x)=x^3 + 2*x^2 - 7',cex.lab=1.5,cex.main=2)
abline(h=0)

nr = function(x, tol = 0.001) {
	#Newton-Raphson iteration for f
	while(abs(f(x)) > tol) {
		x = x - (f(x) / f.prime(x))
		print(x)
	}
	return(x)
}
x=4
nr(x)
abline(a=f(x)-f.prime(x)*x,b=f.prime(x),col=2)

x=-0.1
nr(x)
abline(a=f(x)-f.prime(x)*x,b=f.prime(x),col=2)


###########

#some useful functions
#all.equal ('nearly equal')
0.3==(0.1+0.1+0.1) #false
all.equal(0.3,(0.1+0.1+0.1)) #true

#identical (are these two objects the same?)
1==c(1,2,3) #is element by element
identical(1,c(1,2,3)) #the vector 1 and the vector c(1,2,3) are not the same

#all(), any() useful tests on boolean vector
x=c(1,3,47,-1)
any(x<0)
all(x<0)
 
