
#L8/3rd Prac: Part A Simulation and Statistical Programming

q<-function(n=1) {
  #simulate exp(-|x|)
  X<-log(runif(n))
  Y<-sample(c(-1,1),n,replace=T)
  return(X*Y)
}
q(n=4)

y<-q(1e4); a<-hist(y,100,freq=F)
lines(a$breaks,0.5*exp(-abs(a$breaks)),col=2,lwd=2)

my_rnorm<-function() {
   #simulate X~N(0,1) by rejection
   finished<-FALSE;
   while (!finished) {
      y<-q(n=1);
      u<-runif(n=1)
      p.over.Mq<-exp(-y^2/2+abs(y)-0.5)
      finished<-(u<p.over.Mq)
   }
   return(y)
}
#test it
k<-1000; X<-numeric(k); 
for (i in 1:k) X[i]<-my_rnorm(); 
qqnorm(X); qqline(X)  

###
#vectorising code
#Example: cumulative sum
n<-10; X<-rep(1,n); for (i in 2:10) X[i]<-i+X[i-1]; X
X<-cumsum(1:n); X


#Example: sum x^2 x=1...n
tot<-0; for (x in 1:10) tot<-tot+x^2; tot
sum((1:10)^2)

#using apply()
X<-matrix(c(1,2,3,4),2,2); X
apply(X,1,sum) #sum rows
apply(X,2,max) #sum columns

#how many entries in each column less than 2.5?
apply(X,2,function(x) {sum(x<2.5)})
 

#use apply() to replace for-loops
#Example: simulate n Gamma(a,b)
a<-3; b<-4.1; n<-10; X<-numeric(n)
for (i in 1:n) {
   tot<-0
   for (j in (1:a)) {
        tot<-tot-log(runif(1))/b
   }
   X[i]<-tot
}
X

#or
M<- -log(matrix(runif(a*n),a,n)); X<-apply(M,2,sum); X

## Data
#read the data
hd<-read.table('hellung.txt',header=TRUE)
dim(hd)
head(hd)

#show me the data!
plot(log(hd$conc),hd$diameter)
plot(hd)

## The path, and attach
search()
plot(log(conc),diameter) #an error if hd not attached
attach(hd)
plot(log(conc),diameter)
search()

##
#functions and NA
eg=c(1,2,NA,3,4)
mean(eg)
mean(eg,na.rm=T)

#apply down column
apply(hd,2,max) #2->1 would max over rows

#Subsetting - using boolean->index mapping
mean(diameter[glucose==1])
mean(diameter[glucose==2])
#Subsetting - to create the subset dataframe
hd.g1=hd[glucose==1,]
summary(hd.g1)
summary(hd)

##
#Histograms
hist(diameter,freq=TRUE);
hist(diameter,freq=FALSE);
par(mfrow = c(1,2)); hist(diameter); hist(conc)

n = 100000; x = rnorm(n)
par(mfrow = c(2,2));
hist(x, main = "Sturges")
hist(x, breaks = "Scott", main = "Scott")
hist(x, breaks = sqrt(n),main='sqrt(n)')
hist(x, breaks = seq(from=-5,to=5,length.out=200),main='my breaks')

#a density overlay using lines()
hist(diameter,freq=FALSE);
lines(density(diameter),col=2) 

#it is the same as the more primitive
d=density(diameter)
names(d)
lines(d$x,d$y,col=2)

##
#Boxplots
boxplot(diameter~glucose,xlab='glucose')
 
##
#Barplots
#The oxford FHS data.
ox=read.csv("Oxford_data_anon.csv",header=TRUE)
ox$Result<-factor(ox$Result,levels=c('F','TO','TT','T'))

(ox.tab=table(ox$Result,ox$College))

barplot(ox.tab[1,],cex.names=0.5)
barplot(ox.tab,cex.names=0.5)

##
#regular scatter plots
#plot gives points by default - type='l' option for lines
plot(log(conc), diameter, col=glucose, pch=glucose)
abline(37.806,-1.264)
abline(36.327,-1.264,col=2)

plot(log(conc), diameter, col=glucose, pch=glucose)
curve(37.806-1.264*x, from=8 , to=15, add=T)
curve(36.327-1.264*x, from=8 , to=15, add=T, col=2)